// Canon BubbleJet BJ series (B&W) driver Class
//
// 2001, Philippe Houdoin
// Based on:
// - gsbj10.c from GhostScript
// - LMDriver.cpp from Ryan Lockhart Lexmark 5/7xxxx BeOS driver 
//
// Printers supported
// - don't really know, print scanline is still to be implemented!
// - from documentation, it should be:
//   BJ-10e, BJ-10ex, BJ-10sx, BJ-130, BJ-130e, BJ-200, BJ-200e, BJ-200ex 
// 
// Todo list:
// - dithering bitmap
// - scanline bitmap
// - print scanline
// - add landscape
// - decrease memory & increase speed: multiple partial rendition?
// - gamma correction?

#define DEBUG	1

#include <Debug.h>

#include <stdio.h>
#include <StorageKit.h>

#include "BJ.h"

#ifdef DEBUG
	#include "BitmapWindow.h"
#endif



// Constructor & destructor
// ------------------------

BJ::BJ()
	: Printer()
{
}

BJ::~BJ()
{
	CloseTransport();
}


// Methods
// -------

status_t BJ::PrintPage
	(
	BFile *		spool,
	BMessage *	job_msg,
	int32		page_number,
	int32		page_count
	)
{
	status_t	status;
	char 		text[512];

	status = RasterizePage(spool, job_msg);

#ifdef DEBUG

	// Show bitmap in a window
	sprintf(text, "Printing page %ld/%ld...", page_number, page_count);
	BitmapWindow *	bw = new BitmapWindow(text, page_bitmap);
	bw->Go();

	if ( page_bitmap )
		delete page_bitmap;
#else
	if ( page_number == 1 )
		// Init printer before first page
		Init();

	LoadPage();


	// avance paper 360/360 inches, aka one inch 
	int 	count = 360;
	uint8	byte;
				
	while ( count > 255 )
		{
		transport->Write("\033J\377", 3);
		count -= 255;
		};
	if ( count )
		{
		transport->Write("\033J", 2);
		byte = count;
		transport->Write(&byte, 1);
		};

	// avance print head 120/120 inches, aka one inch to the right
	count = 120;
	sprintf(text, "\033d%c%c", count & 0xFF, count >> 8);
	transport->Write(text, 4); 

	// print a 1 dot line graphic, 360/360 inches long
	int i;
	count = 360*6;	// 6*8 bits on print head (48 dots)
	sprintf(text, "\033[g%c%c%c", count & 0xFF, count >> 8, 16);
	transport->Write(text, 6);

	for ( i = 0; i < count ; i++)
		transport->Write("\001\000\000\000\000\000", 6);
				
	transport->Write("\r", 1);	// scanline end

	EjectPage();
#endif	// #if ! DEBUG

	return status;
}


status_t BJ::Init()
{
	uint8	byte;

	transport->Write("\033[K\002\000\000\044", 7);	// A4--DIP switch defaults
	// transport->Write("\033[K\002\000\004\044", 7);	// letter--factory defaults
	
	// turn off automatic carriage return
	transport->Write("\0335\000", 3);
	
	// Set vertical spacing (yres; in dpi)
	transport->Write("\033[\\\004\000\000\000", 7);
	byte = 360 & 0xFF;
	transport->Write(&byte, 1);
	byte = 360 >> 8;
	transport->Write(&byte, 1);

	// Set vertical lenght of page, in inch
	transport->Write("\033C\00", 3);
	byte = 12;				// 12 inches = A4
	transport->Write(&byte, 1);

	return B_OK;
}



status_t BJ::LoadPage()
{
	return B_OK;
}



status_t BJ::EjectPage()
{
	uint8	byte;

	byte = 0x0c;	// Form Feed
	transport->Write(&byte, 1); 
	return B_OK;
}


status_t BJ::RasterizePage
	(
	BFile *		spool,
	BMessage *	msg
	)
{
	BView * view;
	long picture_count;
	BRect print_rect;
	BRect * pic_rects;
	BPoint * pic_points;
	BRegion * pic_region;
	BPicture ** pictures;
	int32 pic;
	
	print_rect = msg->FindRect("printable_rect");
	
	spool->Read(&picture_count, sizeof(long));
		
	pictures = (BPicture **)malloc(picture_count * sizeof(BPicture *));
	pic_rects = (BRect *)malloc(picture_count * sizeof(BRect));
	pic_points = (BPoint *)malloc(picture_count * sizeof(BPoint));
	pic_region = new BRegion();
	
	for (pic = 0; pic < picture_count; pic++)
	{
		spool->Seek(40 + sizeof(off_t), SEEK_CUR);
		spool->Read(&pic_points[pic], sizeof(BPoint));
		spool->Read(&pic_rects[pic], sizeof(BRect));
		pictures[pic] = new BPicture();
		pictures[pic]->Unflatten(spool);
		pic_region->Include(pic_rects[pic]);
	}
	
	float scale = 360.0/ 72.0;
	
	
	BRect bitmap_rect(0, 0, (pic_region->Frame().right - pic_region->Frame().left) * scale, 
						(pic_region->Frame().bottom - pic_region->Frame().top) * scale);	
	
	view = new BView(bitmap_rect, "", B_FOLLOW_ALL, B_WILL_DRAW);
	page_bitmap = new BBitmap(bitmap_rect, B_GRAY8, true);
	 
	page_bitmap->AddChild(view);
	
	view->Window()->Lock();
	view->SetScale(scale);
	
	// view->SetHighColor(255,255,255);
	// view->FillRect(view->Bounds());
	view->MovePenTo(0,0);
		
	for (pic = 0; pic < picture_count; pic++)
		view->DrawPicture(pictures[pic]);
		
	return B_OK;
}
