#include "BitmapWindow.h"

BitmapWindow::BitmapWindow(const char * name, BBitmap * bitmap)
	: BWindow(BRect(0,0,600,600), name,
	B_DOCUMENT_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL,
	B_ASYNCHRONOUS_CONTROLS)
{
	exit_sem 	= create_sem(0, "BitmapWindow");
	result 		= B_OK;
	
	BRect	r 	= Bounds();
	r.right 	-= B_V_SCROLL_BAR_WIDTH;
	r.bottom 	-= B_H_SCROLL_BAR_HEIGHT;
	
	BitmapView *	bv 	= new BitmapView(r, bitmap);
	BScrollView *	sv 	= new BScrollView("bitmap_scroll", bv,
										B_FOLLOW_ALL_SIDES, B_WILL_DRAW, true, true);
	AddChild(sv);
}


BitmapWindow::~BitmapWindow()
{
	delete_sem(exit_sem);
}




bool BitmapWindow::QuitRequested()
{
	release_sem(exit_sem);
	return true;
}

status_t BitmapWindow::Go()
{
	MoveTo(60,60);
	Show();
	acquire_sem(exit_sem);
	Lock();
	Quit();

	return result;
}


// -----------------------------



BitmapView::BitmapView(BRect frame, BBitmap * bm)
	: BView(frame, "BitmapView",
			B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS)
{
	bitmap = bm;
}

void BitmapView::AttachedToWindow(void)
{
	SetScrollbars();
	SetViewColor(B_TRANSPARENT_32_BIT);
}

void BitmapView::FrameResized(float width, float height)
{
	SetScrollbars();
}

void BitmapView::Draw(BRect invalid)
{
	if( bitmap )
		DrawBitmap(bitmap, invalid, invalid);
}


void BitmapView::SetScrollbars(void)
{
	BRect 			bounds;
	BScrollBar *	sb;

	bounds = Bounds();
	float myPixelWidth = bounds.Width();
	float myPixelHeight = bounds.Height();
	float maxWidth = 1, maxHeight = 1;

	if( bitmap != NULL)
		{
		// get max size of image
		Window()->Lock();
		maxWidth 	= bitmap->Bounds().right;
		maxHeight 	= bitmap->Bounds().bottom;
		Window()->Unlock();
		};
		
	float propW = myPixelWidth/maxWidth;
	float propH = myPixelHeight/maxHeight;
	
	float rangeW = maxWidth - myPixelWidth;
	float rangeH = maxHeight - myPixelHeight;

	if(rangeW < 0) rangeW = 0;
	if(rangeH < 0) rangeH = 0;

	if ((sb=ScrollBar(B_HORIZONTAL))!=NULL) {
		sb->SetProportion(propW);
		sb->SetRange(0,rangeW);
		// Steps are 1/8 visible window for small steps
		//   and 1/2 visible window for large steps
		sb->SetSteps(myPixelWidth / 8.0, myPixelWidth / 2.0);
	} 

	if ((sb=ScrollBar(B_VERTICAL))!=NULL) {
		sb->SetProportion(propH);
		sb->SetRange(0,rangeH);
		// Steps are 1/8 visible window for small steps
		//   and 1/2 visible window for large steps
		sb->SetSteps(myPixelHeight / 8.0, myPixelHeight / 2.0);
	}
}

